var register;          // static registers to store variables
var system_window;     // system window
var system_big_font;   // system font, big
var system_small_font; // system font, small
var system_pointer;    // system pointer
var system_music;      // script level music for certain events
var system_sound;      // script level sound for certain events
var sound_dat;         // sound effect database
var verge_palette;     // verge palette

////////////////////////////////////////////////////////////////////

function initSystem()
{
  register = new Array(8000);
  system_window = LoadWindowStyle("verge.rws");
  system_font_small = LoadFont("small.rfn");
  system_font_big = LoadFont("big.rfn");
  system_pointer = LoadImage("pointer.png");
  //LoadVergePal();
}

////////////////////////////////////////////////////////////////////

function initSoundSystem()
{
  var main_sfx;
  var data_dump;
  var total_sounds;
  var offset = 0;
  var total_sounds_struct;
  
  sound_dat = new Array();
  main_sfx  = OpenRawFile("main.sfx");
  data_dump = main_sfx.read(main_sfx.getSize());
  
  // get the number of sounds
  total_sounds_struct = grabStringFromDat(data_dump, offset);
  total_sounds = parseInt(total_sounds_struct.text);
  offset = total_sounds_struct.offset;
  
  // read the sounds into sound_dat :)
  for (var i=0; i<total_sounds; i++) {
  
    var sound_file = grabStringFromDat(data_dump, offset);
    sound_dat[i]   = sound_file.text;
    offset         = sound_file.offset;
  }
}

////////////////////////////////////////////////////////////////////

function playEffect(sound_index)
{
  if (sound_index >= sound_dat.length || sound_index < 0) {
    Debug("Invalid sound_index range! Sound Index selected: " + sound_index);
    Exit();
  }
  
  system_sound = LoadSound(sound_dat[sound_index]);
  system_sound.play(false);
}

////////////////////////////////////////////////////////////////////

function ClearScreen()
{
  var black = CreateColor(0, 0, 0);
  ApplyColorMask(black);
}

////////////////////////////////////////////////////////////////////

function Debug(string)
{
  ClearScreen();
  while(AreKeysLeft()) 
    GetKey();
    
  while(!AreKeysLeft())
  {
    system_font_small.drawText(0, 0, string);
    FlipScreen();
  }
}

////////////////////////////////////////////////////////////////////

function FadeOut(milliseconds)
{
  var start = GetTime();
  var bg = GrabImage(0, 0, 320, 200);
  black = CreateColor(0, 0, 0);
  
  while (GetTime() - start < milliseconds)
  {
    bg.blit(0, 0);
    black.alpha = (GetTime() - start) * 255 / milliseconds;
    ApplyColorMask(black);
    FlipScreen();
    current = GetTime();
  }
  
  // put a copy of the background back in if needed
  bg.blit(0, 0);
}

////////////////////////////////////////////////////////////////////

function FadeIn(milliseconds)
{
  var start = GetTime();
  var bg = GrabImage(0, 0, 320, 200);
  black = CreateColor(0, 0, 0);
  
  while (GetTime() - start < milliseconds)
  {
    bg.blit(0, 0);
    black.alpha = 255 - ((GetTime() - start) * 255 / milliseconds);
    ApplyColorMask(black);
    FlipScreen();  
    current = GetTime();
  }
  
  // put a copy of the background back in if needed
  bg.blit(0, 0);
}

////////////////////////////////////////////////////////////////////

function Delay(milliseconds)
{
  var start = GetTime();
  while (GetTime() - start < milliseconds)
    ;
}

////////////////////////////////////////////////////////////////////

function grabStringFromDat(dat_file, offset)
{

  var current = offset;
  var text = "";
  var done = false;
  
  current = skipWhiteSpace(dat_file, current);
  while (1)
  {
    // check if the pointer has reached the end of the array
    if (current >= dat_file.length)
      break;
   

    // acquire character and check if it's valid
    var c = String.fromCharCode(dat_file[current]);
    if (c == '\n' || c == '\r' || c == ' ')
      break;
    else
    {
      text += c;
      current++;
    }
  }
  
  // return the info via an object
  var info = new Object();
  info.offset = current;
  info.text = text;
  return info;
}

////////////////////////////////////////////////////////////////////

function skipWhiteSpace(dat_file, offset)
{
  var current = offset;
  var done = false;
  
  while (1)
  {
    // check if the pointer has reached the end of the array
    if (current >= dat_file.length)
      break;
  
    // acquire character and check if it's valid
    var c = String.fromCharCode(dat_file[current]);
    if (c == '\n' || c == '\r' || c == ' ')
      current++;
    else
      break;
  }
  
  return current;
}

////////////////////////////////////////////////////////////////////

function LoadVergePal()
{
  var verge_pal = OpenRawFile("verge.pal");
  var data_dump = verge_pal.read(768);
  
  verge_palette = new Array(256);
  for (var i=0; i<256; i++) {
    verge_palette[i] = CreateColor(data_dump[i*3] << 2, data_dump[i*3 + 1] << 2, data_dump[i*3 + 2] << 2);
  }
}

////////////////////////////////////////////////////////////////////

function LoadCR2(filename)
{
  // CR2 data format
  // 96x96 of raw data, referencing to the verge palette

  var cr2_handle = OpenRawFile(filename)
  var data_dump  = cr2_handle.read(cr2_handle.getSize());
  //var cr2_image  = CreateSurface();
  var cr2_image  = GrabSurface(0, 0, 96, 96);

  for (var y=0; y<96; y++)
    for (var x=0; x<96; x++) {
      var c = data_dump[y*96 + x];
      cr2_image.setPixel(x, y, verge_palette[c]);
    }
  
  return cr2_image.createImage();
}

////////////////////////////////////////////////////////////////////

function MapChange(mapname, leader, x, y)
{
  FadeOut(800);
  ChangeMap(mapname);
  
  SetPersonX(leader, x * 16 + 8);
  SetPersonY(leader, y * 16 + 8);
  SetPersonLayer(leader, 0);
  
  RenderMap();
  FadeIn(800);
}

////////////////////////////////////////////////////////////////////